/*

    qmailalizer - a qmail-send log analysis program

    Copyright (C) 2001  Pichler Wolfgang  (wpichler@dialog-gruppe.at)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version, and provided that the above
    copyright and permission notice is included with all distributed
    copies of this or derived software.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

    This software uses the gd graphics library, which is copyright by
    Quest Protein Database Center, Cold Spring Harbor Labs.  Please
    see the documentation supplied with the library for additional
    information and license terms, or visit www.boutell.com/gd/ for the
    most recent version of the library and supporting documentation.


*/

#include "message.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "stuff.h"

MHandler::MHandler ()
{
  loguser = NULL;
  mWurzel = NULL;
  lmessage = NULL;
}

MHandler::~MHandler ()
{ Message *aktmessage,*lmessage;
  Deliveries *aktdeliverie,*ldeliverie;
  
  // Die Strukturen wieder freigeben
  // Als erstes die Messages  + deliveries freigeben
  aktmessage = mWurzel;
  while (aktmessage != NULL) {
    aktdeliverie = aktmessage->deliveries;
    while (aktdeliverie != NULL) {
      ldeliverie = aktdeliverie->next;
      // Die einzelnen chars wieder freigeben
      if (aktdeliverie->recipient != NULL)
        free(aktdeliverie->recipient);
      if (aktdeliverie->zeitpunkt != NULL)
        free(aktdeliverie->zeitpunkt);
      if (aktdeliverie->servermessage != NULL)
        free(aktdeliverie->servermessage);
      if (aktdeliverie->state != NULL)
        free(aktdeliverie->state);
      free (aktdeliverie);
      aktdeliverie = ldeliverie;
    }
    lmessage = aktmessage->next;
    if (aktmessage->sender != NULL)
      free (aktmessage->sender);
    if (aktmessage->recipient != NULL)
      free (aktmessage->recipient);
    if (aktmessage->startzeit != NULL)
      free (aktmessage->startzeit);
    if (aktmessage->endzeit != NULL)
      free (aktmessage->endzeit);
    free (aktmessage);
    aktmessage = lmessage;
  }
  mWurzel = NULL;
}

int MHandler::deliveriestatus (int local,int maxlocal,int remote,int maxremote)
{
  if (aktdeliverie != NULL) {
    aktdeliverie->local = local;
    aktdeliverie->maxlocal = maxlocal;
    aktdeliverie->remote = remote;
    aktdeliverie->maxremote = maxremote;
  }
  return 0;
}

int MHandler::newdeliverie (unsigned long dnr,unsigned long msgnr,char *recipient,char *localremote,struct tm *datum)
{ Deliveries *ndeliverie;
  Message *amessage;
  Deliveries *adeliverie;
  bool notparse;
  char *user;
  int atpos,spos;
  
  // Als erstes die dafr zustndige Nachricht suchen
  // Wenn aber der Empfnger der loguser ist dann nicht weiter verarbeiten
  user = NULL;
  if (recipient != NULL) {
    atpos = strpos(recipient,'@');
    spos = strpos(recipient,'-');
    if (spos > atpos) spos = 0;
    if (spos > 0) spos++;
    user = (char *) malloc(atpos-spos+1);
    strncpy (user,recipient+spos,atpos-spos);
    user[atpos-spos] = 0;
  }
 if ((loguser != NULL) && (user != NULL)) {
   if (strcmp(user,loguser)==0) {
     notparse = true;
   }
   else {
     notparse = false;
   }
  }
  else
    notparse = false;
 if (user != NULL) free(user);
 if (notparse == false) {
  if ((amessage = getmessage(msgnr)) != NULL) {
    ndeliverie = new_deliverie();
    ndeliverie->dnr = dnr;
    ndeliverie->msgnr = msgnr;
    ndeliverie->recipient = (char *)malloc(strlen(recipient)+1);
    strlower (recipient);
    strcpy (ndeliverie->recipient,recipient);
    if (amessage->recipient==NULL)
    {
      amessage->recipient = (char *)malloc(strlen(recipient)+1);
      strcpy (amessage->recipient,recipient);    
    }
    if (strcmp(localremote,"remote") == 0)
      ndeliverie->tolocal = false;
    else
      ndeliverie->tolocal = true;
    ndeliverie->zeitpunkt = new (struct tm);
    memcpy (ndeliverie->zeitpunkt,datum,sizeof(struct tm));
    aktdeliverie = ndeliverie;
    // Jetzt noch in die Struktur einhngen
    if ((adeliverie = amessage->deliveries) == NULL)
      amessage->deliveries = ndeliverie;
    else
    {
      while (adeliverie->next != NULL) {
        adeliverie = adeliverie->next;
      }
      adeliverie->next = ndeliverie;
    }
  }
 }
  return 0;
}

int MHandler::deliveriestate (unsigned long udnr, char *dstate,char *servermessage,struct tm *datum)
{ Deliveries *adeliverie;

  adeliverie = getdeliverie(udnr);      
  if (adeliverie != NULL) {
    // Existiert - also die Werte eintragen
    // Wenn der dstate auf success ist dann ist die Nachricht abgeschlossen
    if (strcmp(dstate,"success") == 0)
      endmessage(adeliverie->msgnr,datum);
    adeliverie->state = (unsigned char *)malloc(strlen(dstate)+1);
    strcpy((char *)adeliverie->state,dstate);
    adeliverie->servermessage = (unsigned char *)malloc(strlen(servermessage)+1);
    strcpy((char *)adeliverie->servermessage,servermessage);
  }
  return 0;
}

int MHandler::endmessage (unsigned long msgnr, struct tm *datum)
{ Message *amessage;

  if ((amessage = getmessage(msgnr)) != NULL)
  {
    amessage->endzeit = new(struct tm);
    memcpy (amessage->endzeit,datum,sizeof(struct tm));
    amessage->endmsg = true;
  }
  return 0;
}

int MHandler::infomessage(unsigned long msgnr,unsigned long bytes,char *sender,struct tm *datum)
{ Message *amessage;

  if ((amessage = getmessage(msgnr)) == NULL)
  {
    // Existiert noch nicht
    // Also erzeugen
    amessage = newmessage(msgnr,datum);
  }
  // Hier die Daten schreiben
  amessage->bytes = bytes;
  amessage->sender = (char *) malloc(strlen(sender)+1);
  strlower (sender);
  strcpy (amessage->sender,sender);
  return 0;
}

Message *MHandler::newmessage(unsigned long msgnr,struct tm *datum)
{ Message *amessage;
  Message *nmessage;

  if ((amessage = getmessage(msgnr)) != NULL) {
    // Da wurde anscheinend einmal vergessen das end msg zu schreiben, also selbst schlie0en
    endmessage (msgnr,datum);
    amessage = NULL;
  }
  nmessage = new_message ();
  nmessage->startzeit = new(struct tm);
  memcpy (nmessage->startzeit,datum,sizeof(struct tm));
  nmessage->msgnr = msgnr;
  // und ganz hinten anhngen
  amessage = lmessage;
  if (amessage != NULL) {
    amessage->next = nmessage; 
    nmessage->last = amessage;
    lmessage = nmessage;
  }
  else
  {
    // Wurzel
    mWurzel = nmessage;
    lmessage = nmessage;
  }
  return nmessage;
}

Message *MHandler::new_message ()
{ Message *nelement = new(Message);

  // Standard Werte setzen
  nelement->next = NULL;
  nelement->last = NULL;
  nelement->sender = NULL;
  nelement->recipient = NULL;
  nelement->startzeit = NULL;
  nelement->endzeit = NULL;
  nelement->msgnr = 0;
  nelement->bytes = 0;
  nelement->endmsg = false;
  nelement->deliveries = NULL;
  return nelement;
}

Message *MHandler::getmessage (unsigned long msgnr)
{ Message *aelement;

  if (lmessage == NULL)
  {
    aelement = mWurzel;
    while (aelement != NULL) {
      if ((aelement->msgnr == msgnr) && (aelement->endmsg == false))
        break;
      aelement = aelement->next;
    }
  }
  else
  {
    aelement = lmessage;
    while (aelement != NULL) {
      if ((aelement->msgnr == msgnr) && (aelement->endmsg == false))
        break;
      aelement = aelement->last;
    }
  }
  return aelement;
}

Deliveries *MHandler::new_deliverie ()
{ Deliveries *nelement = new(Deliveries);
  
  nelement->next = NULL;
  nelement->recipient = NULL;
  nelement->zeitpunkt = NULL;
  nelement->dnr = 0;
  nelement->servermessage = NULL;
  nelement->state = NULL;
  nelement->msgnr=0;
  nelement->local = 0;
  nelement->maxlocal = 0;
  nelement->remote = 0;
  nelement->maxremote = 0;
  nelement->tolocal = false;
  return nelement;
}

Deliveries *MHandler::getdeliverie (unsigned long dnr)
{ Deliveries *adeliverie=NULL;
  Message *amessage;
  
  if (lmessage == NULL)
  {
    amessage = mWurzel;
    while (amessage != NULL) {
      if ((adeliverie = amessage->deliveries) != NULL) {
        while (adeliverie != NULL) { 
          if (adeliverie->dnr == dnr)
     	  break;
          adeliverie = adeliverie->next;
        }
      }  
      if (adeliverie != NULL)
        break;
      amessage = amessage->next;
    }
  }
  else
  {
    amessage = lmessage;
    while (amessage != NULL) {
      if ((adeliverie = amessage->deliveries) != NULL) {
        while (adeliverie != NULL) { 
          if (adeliverie->dnr == dnr)
       	    break;
          adeliverie = adeliverie->next;
        }
      }  
      if (adeliverie != NULL)
        break;
      amessage = amessage->last;
    }
  }
  return adeliverie;
}
