/*

    qmailalizer - a qmail-send log analysis program

    Copyright (C) 2001  Pichler Wolfgang  (wpichler@dialog-gruppe.at)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version, and provided that the above
    copyright and permission notice is included with all distributed
    copies of this or derived software.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

    This software uses the gd graphics library, which is copyright by
    Quest Protein Database Center, Cold Spring Harbor Labs.  Please
    see the documentation supplied with the library for additional
    information and license terms, or visit www.boutell.com/gd/ for the
    most recent version of the library and supporting documentation.


*/

#include "xmloutput.h"
#include <string.h>

XMLOutput::XMLOutput ()
{
  // Hier wird die xml Structur initialisiert
  InitXML ();
}

XMLOutput::~XMLOutput ()
{
  // Hier mu der reservierte Speicher fr die XML Daten wieder Freigegeben werden
  FreeXML ();
}

void XMLOutput::AddFile (char *filename,bool processed,char *zeit)
{ xmlNodePtr file;

  file = xmlNewChild(NFiles,NULL,(xmlChar *)"FILE",(xmlChar *)filename);
  if (processed)
    xmlSetProp (file,(xmlChar *)"processed",(xmlChar *)"true");
  else
    xmlSetProp (file,(xmlChar *)"processed",(xmlChar *)"false");  
  xmlSetProp (file,(xmlChar *)"datetime",(xmlChar *)zeit);

}

void XMLOutput::AddMessage (Message *message)
{ xmlNodePtr mnode,deliveries,deliverie;
  unsigned char *temp,*temp1;
  Deliveries *aktdeliverie;
  int i;
   
  temp = (unsigned char *) malloc(60);
  temp = ltoa (message->msgnr,temp,10);
  mnode = xmlNewChild(NMessages,NULL,(xmlChar *)"MESSAGE",(xmlChar *)temp);  
  xmlSetProp (mnode,(xmlChar *)"sender",(xmlChar *)message->sender);
  xmlSetProp (mnode,(xmlChar *)"recipient",(xmlChar *)message->recipient);
  temp = ltoa (message->bytes,temp,10);
  xmlSetProp (mnode,(xmlChar *)"bytes",(xmlChar *)temp);
  temp1 = temp;
  temp = (unsigned char *)asctime(message->startzeit);
  if (temp != NULL)
    temp[strlen((char*)temp)-1] = 0;
  else
  {
    temp = temp1;
    temp[0] = 0;
  }
  xmlSetProp (mnode,(xmlChar *)"startzeit",(xmlChar *)temp);
  temp1 = temp;
  temp = (unsigned char *)asctime(message->endzeit);
  if (temp != NULL)
    temp[strlen((char*)temp)-1] = 0;
  else
  {
    temp = temp1;
    temp[0] = 0;
  }
  temp[strlen((char *)temp)-1] = 0;
  xmlSetProp (mnode,(xmlChar *)"endzeit",(xmlChar *)temp);
  aktdeliverie = message->deliveries;
  temp = ltoa (message->msgnr,temp,10);
  deliveries = xmlNewChild(mnode,NULL,(xmlChar *)"DELIVERIES",(xmlChar *)temp);
  i = 0;
  while (aktdeliverie != NULL) {
    i++;
    // Das hier ist fr die delivery Versuche
    temp = ltoa (aktdeliverie->dnr,temp,10);
    deliverie = xmlNewChild(deliveries,NULL,(xmlChar *)"DELIVERIE",(xmlChar *)temp);
    temp = itoa (i,temp,10);
    xmlSetProp (deliverie,(xmlChar *)"attemp",(xmlChar *)temp);
    temp1 = temp;
    temp = (unsigned char *)asctime(aktdeliverie->zeitpunkt);
    if (temp != NULL)
      temp[strlen((char*)temp)-1] = 0;
    else
    {
      temp = temp1;
      temp[0] = 0;
    }
    xmlSetProp (deliverie,(xmlChar *)"zeitpunkt",(xmlChar *)temp);
    xmlSetProp (deliverie,(xmlChar *)"smessage",(xmlChar *)aktdeliverie->servermessage);
    xmlSetProp (deliverie,(xmlChar *)"state",(xmlChar *)aktdeliverie->state);
    xmlSetProp (deliverie,(xmlChar *)"recipient",(xmlChar *)aktdeliverie->recipient);
    if (aktdeliverie->tolocal)
      xmlSetProp (deliverie,(xmlChar *)"local",(xmlChar *)"true");
    else
      xmlSetProp (deliverie,(xmlChar *)"local",(xmlChar *)"false");
    aktdeliverie = aktdeliverie->next;
  }
}

void XMLOutput::AddDomain (Domains *domainptr)
{ xmlNodePtr domain;
  xmlNodePtr duser;
  xmlNodePtr year;
  xmlNodePtr month;
  int i;
  unsigned char *temp = (unsigned char *)malloc(60);
  unsigned char *temp1;
  Members *member;
  
  domain = xmlNewChild(NDomains,NULL,(xmlChar *)"DOMAIN",(xmlChar *)domainptr->domainname);
  xmlSetProp (domain,(xmlChar *)"domain",(xmlChar *)domainptr->domainname);
  // Der Link gehrt noch bercksichtig
  xmlSetProp (domain,(xmlChar *)"link",(xmlChar *)domainptr->link);
  xmlSetProp (domain,(xmlChar *)"image",(xmlChar *)domainptr->filename);
  temp = ltoa (domainptr->anzahlm,temp,10);
  xmlSetProp (domain,(xmlChar *)"summessages",(xmlChar *)temp);
  temp = dtoa (domainptr->bytes,temp,10);
  xmlSetProp (domain,(xmlChar *)"sumbytes",(xmlChar *)temp);
  temp = ltoa (domainptr->incoming,temp,10);
  xmlSetProp (domain,(xmlChar *)"incoming",(xmlChar *)temp);
  temp = ltoa (domainptr->outgoing,temp,10);
  xmlSetProp (domain,(xmlChar *)"outgoing",(xmlChar *)temp);
  temp = dtoa (domainptr->incbytes,temp,10);
  xmlSetProp (domain,(xmlChar *)"incbytes",(xmlChar *)temp);
  temp = dtoa (domainptr->outbytes,temp,10);
  xmlSetProp (domain,(xmlChar *)"outbytes",(xmlChar *)temp);
  temp = ltoa (domainptr->local,temp,10);
  xmlSetProp (domain,(xmlChar *)"local",(xmlChar *)temp);
  temp = dtoa (domainptr->localbytes,temp,10);
  xmlSetProp (domain,(xmlChar *)"localbytes",(xmlChar *)temp);
  temp1 = temp;
  temp = (unsigned char *)asctime(domainptr->start);
  if (temp != NULL)
    temp[strlen((char*)temp)-1] = 0;
  else
  {
    temp = temp1;
    temp[0] = 0;
  }
  xmlSetProp (domain,(xmlChar *)"startzeit",(xmlChar *)temp);
  temp = temp1;
  temp1 = temp;
  temp = (unsigned char *)asctime(domainptr->end);
  if (temp != NULL)
    temp[strlen((char*)temp)-1] = 0;
  else
  {
    temp = temp1;
    temp[0] = 0;
  }
  xmlSetProp (domain,(xmlChar *)"endzeit",(xmlChar *)temp);
  temp = temp1;
  temp = ltoa (domainptr->sec,temp,10);
  xmlSetProp (domain,(xmlChar *)"sec",(xmlChar *)temp);
  // Hier kommen nun die Mitglieder dieser domain
  member = domainptr->members;
  while (member != NULL) {
    // Den Sub Eintrag erstellen
    duser = xmlNewChild(domain,NULL,(xmlChar *)"MEMBER",(xmlChar *)member->name);
    // Und die Attribute setzen
    temp = ltoa (member->anzahlm,temp,10);
    xmlSetProp (duser,(xmlChar *)"summessages",(xmlChar *)temp);
    temp = ltoa (member->bytes,temp,10);
    xmlSetProp (duser,(xmlChar *)"sumbytes",(xmlChar *)temp);
    temp = ltoa (member->incoming,temp,10);
    xmlSetProp (duser,(xmlChar *)"incoming",(xmlChar *)temp);
    temp = ltoa (member->outgoing,temp,10);
    xmlSetProp (duser,(xmlChar *)"outgoing",(xmlChar *)temp);
    temp = ltoa (member->incbytes,temp,10);
    xmlSetProp (duser,(xmlChar *)"incbytes",(xmlChar *)temp);
    temp = ltoa (member->outbytes,temp,10);
    xmlSetProp (duser,(xmlChar *)"outbytes",(xmlChar *)temp);
    temp = ltoa (member->local,temp,10);
    xmlSetProp (duser,(xmlChar *)"local",(xmlChar *)temp);
    temp = ltoa (member->localbytes,temp,10);
    xmlSetProp (duser,(xmlChar *)"localbytes",(xmlChar *)temp);
    temp1 = temp;
    temp = (unsigned char *)asctime(member->start);
    if (temp != NULL)
      temp[strlen((char*)temp)-1] = 0;
    else
    {
      temp = temp1;
      temp[0] = 0;
    }
    xmlSetProp (duser,(xmlChar *)"startzeit",(xmlChar *)temp);
    temp = temp1;
    temp1 = temp;
    temp = (unsigned char *)asctime(member->end);
    if (temp != NULL)
      temp[strlen((char*)temp)-1] = 0;
    else
    {
      temp = temp1;
      temp[0] = 0;
    }
    xmlSetProp (duser,(xmlChar *)"endzeit",(xmlChar *)temp);
    temp = temp1;
    temp = ltoa (member->sec,temp,10);
    xmlSetProp (duser,(xmlChar *)"sec",(xmlChar *)temp);
    member = member->next;
  }
  // Hier kommt der Jahres Usage Bericht
  {
    // Den Sub Eintrag erstellen
    temp = itoa (domainptr->relyear,temp,10);
    year = xmlNewChild(domain,NULL,(xmlChar *)"YEAR",(xmlChar *)temp);
    // Fr jedes Monat einen Child
    for (i=0;i<12;i++) {
      temp = itoa (i,temp,10);
      month = xmlNewChild(year,NULL,(xmlChar *)"MONTH",(xmlChar *)temp);
      temp = ltoa (domainptr->monthusage[i],temp,10);
      xmlSetProp (month,(xmlChar *)"mails",(xmlChar *)temp);
      temp = ltoa (domainptr->monthbytes[i],temp,10);
      xmlSetProp (month,(xmlChar *)"bytes",(xmlChar *)temp);
    }
  }
  free (temp);
}

void XMLOutput::SaveResult (char *FileName)
{
//  xmlSetDocCompressMode (doc,9);      
  xmlSaveFile (FileName,doc);
}

void XMLOutput::InitXML ()
{ time_t aktzeit;
     
  doc = xmlNewDoc((xmlChar *) "1.0");
  doc->charset = XML_CHAR_ENCODING_8859_1;
  doc->children = xmlNewDocNode(doc, NULL,(xmlChar *) "RESULTS", NULL);
  aktzeit = time(NULL);
  xmlSetProp (doc->children,(xmlChar *)"zeitpunkt",(xmlChar *)ctime(&aktzeit));
  // Das hier erstellt die einzelnen Datei Eintrge
  NFiles = xmlNewChild(doc->children,NULL,(xmlChar *)"FILES",NULL);

  // Das hier ist fr jede Message die noch nicht das Ende erreicht hat
  NMessages = xmlNewChild(doc->children,NULL,(xmlChar *)"OPENMESSAGES",NULL);
  
  // Hier kommen die Zur Zeit schon kalkulierten Ergebnisse
  // Aufgeteilt auf die Domains
  NDomains = xmlNewChild(doc->children,NULL,(xmlChar *)"DOMAINS",NULL);
  // Das Bild mu noch generiert werden
  xmlSetProp (NDomains,(xmlChar *)"image",(xmlChar *)"domains.png");
}

void XMLOutput::FreeXML ()
{
  xmlCleanupParser ();
  xmlFreeDoc (doc);
}
